<?php

namespace ePHPic\Autload;

/**
 * Class Autoloader
 *
 * A simple autoloader for loading PHP classes based on a defined mapping and base directory.
 *
 * @package ePHPic\Autload
 */
class Autoloader
{
    /**
     * @var string The base directory for the autoloader.
     */
    private static $baseDir = "/var/www/html";

    /**
     * @var array A mapping of class names to file paths.
     */
    private static $map = [];

    /**
     * Resolves the full file path for a given class name.
     *
     * This method checks if the class is in the map. If it is, it constructs the file path
     * based on the mapping. If not, it attempts to resolve the class name by removing the
     * namespace and constructing the path based on the class name.
     *
     * @param string $class The fully qualified class name.
     * @param string|null $path An optional additional path to append.
     * @return string The resolved file path for the class.
     */
    public static function resolve($class, $path = null)
    {
        if(isset(static::$map[$class]))
        {
            return static::$baseDir . "/" . static::$map[$class] . ($path ? "/$path" : "") . ".php";
        }
        $lastBackslashPos = strrpos($class, '\\');
        if ($lastBackslashPos !== false) {
            $path = substr($class, $lastBackslashPos+1, strlen($class) - 1) . ($path ? "/$path" : "");
            $class = substr($class, 0, $lastBackslashPos);
            return static::resolve($class, $path);
        }

        return static::$baseDir . "/" . str_replace("\\","/",$class) . ($path ? "/$path" : "") . ".php";
    }

    /**
     * Autoloads a class file based on the class name.
     *
     * This method resolves the file path for the class and includes the file if it exists.
     *
     * @param string $class The fully qualified class name to autoload.
     * @return void
     */
    public static function autoload($class)
    {
        $filePath = static::resolve($class);
        if(@file_exists($filePath))
        {
            include_once $filePath;
        }
    }

    /**
     * Registers the autoloader with the SPL autoload stack.
     *
     * This method allows the autoloader to be registered so that it can be used
     * to automatically load classes when they are referenced.
     *
     * @param string $baseDir The base directory for the autoloader.
     * @param array $map An associative array mapping class names to file paths.
     * @return void
     */
    public static function register($baseDir, $map)
    {
        spl_autoload_register(
            [static::class, "autoload"], 
            true, 
            true
        );
    }
}